from django.contrib import admin
from contacts.models import (
    Contact, ContactAddress, Telephone, Address, Email, City)


site = admin.AdminSite('contactbook')


class ContactAddressInline(admin.TabularInline):
    model = ContactAddress
    extra = 2

class TelephoneInline(admin.TabularInline):
    model = Telephone
    extra = 2

class EmailInline(admin.TabularInline):
    model = Email
    extra = 2


class CityAdmin(admin.ModelAdmin):
    list_display = ('name', 'state_abbrev', 'country')
    ordering = ('name',)


class AddressAdmin(admin.ModelAdmin):
    fieldsets = (
        (None, {'fields': ('street', ('number', 'complement'))}),
        (None, {'fields': ('neighbourhood', 'city')}),
    )
    list_display = ('city', 'number', 'complement', 'street', 'neighbourhood')
    list_editable = ('number', 'complement', 'street', 'neighbourhood')
    search_fields = (
        '=number',
        'street',
        'neighbourhood',
        '=city__name',
        '=city__state_abbrev',
        '=city__country',
        '^contactaddress__contact__first_name',
        'contactaddress__contact__surname',
        '=contactaddress__contact__nickname',
    )
    ordering = ('city',)
    inlines = [ContactAddressInline]
    save_as = True
    save_on_top = True


class TelephoneAdmin(admin.ModelAdmin):
    list_display = ('id', 'number', 'kind', 'ok_to_call_now', 'contact')
    list_editable = ('number', 'kind')
    list_filter = ('kind',)
    list_select_related = True
    search_fields = (
        'number',
        '^contact__first_name',
        'contact__surname',
        '=contact__nickname',
    )
    save_as = True


class EmailAdmin(admin.ModelAdmin):
    list_display = ('id', 'address', 'kind', 'contact')
    list_editable = ('address', 'kind')
    list_filter = ('kind',)
    list_select_related = True
    search_fields = (
        'address',
        '^contact__first_name',
        'contact__surname',
        '=contact__nickname',
    )
    save_as = True


class ContactAdmin(admin.ModelAdmin):
    date_hierarchy = 'birthday'
    fieldsets = (
        ('Identification', {
            'fields': ('group', ('first_name', 'surname'),
                       ('nickname', 'treatment'), 'slug'),
        }),
        ('Contact info', {'fields': ('primary_email', 'primary_telephone')}),
        ('Extra info', {
            'fields': (('site_url', 'site_title'),
                       ('birthday', 'profession', 'hometown')),
            'classes': ('collapse',)
        }),
    )
    list_editable = ('primary_email', 'primary_telephone')
    list_display = ('slug', 'first_name', 'surname',
                    'birthday', 'site_link',
                    'primary_email', 'primary_telephone')
    list_filter = ('group',)
    search_fields = (
        '^first_name',
        'surname',
        '=nickname',
        '=primary_email',
        '=primary_telephone',
        'site_url',
        'profession',
        'birthday',
        'contactaddress__address__street',
        'contactaddress__address__neighbourhood',
        '=contactaddress__address__city__name',
        '=contactaddress__address__city__state_abbrev',
        '=contactaddress__address__city__country',)
    prepopulated_fields = {
        'slug': ('group', 'treatment', 'first_name', 'nickname', 'surname')}
    ordering = ('first_name',)
    save_as = True
    save_on_top = True
    inlines = [ContactAddressInline, TelephoneInline, EmailInline]


site.register(Contact, ContactAdmin)
site.register(Address, AddressAdmin)
site.register(Telephone, TelephoneAdmin)
site.register(Email, EmailAdmin)
site.register(City, CityAdmin)
