import datetime
from django.db import models


MIN_TIME = datetime.time.min
MAX_TIME = datetime.time(23, 59, 59)

class Contact(models.Model):
    GROUP_CHOICES = (
        ('fr', 'friends'),
        ('fm', 'family'),
        ('cw', 'coworkers'),
        ('cl', 'coleagues'),
        ('dt', 'dates'),
        ('bp', 'business partners'),
        ('pp', 'project partners'),
        ('sp', 'services providers'),
        ('ot', 'other'),
    )
    # identification
    slug = models.SlugField(primary_key=True)
    first_name = models.CharField(max_length=50)
    surname = models.CharField(max_length=50)
    treatment = models.CharField(max_length=20, blank=True,
                                 help_text='Mr, Mrs, etc')
    nickname = models.CharField(max_length=50, blank=True)

    # getting in contact
    primary_email = models.EmailField(blank=True)
    primary_telephone = models.CharField(max_length=20, blank=True)
    site_url = models.URLField(verify_exists=False, blank=True)
    site_title = models.CharField(max_length=100, blank=True)

    # more about him
    birthday = models.DateField(null=True, blank=True)
    profession = models.CharField(max_length=50, blank=True)
    hometown = models.ForeignKey('City', null=True, blank=True)
    group = models.CharField(max_length=2, choices=GROUP_CHOICES)
    addresses = models.ManyToManyField('Address', through='ContactAddress')

    def __unicode__(self):
        if self.nickname:
            return u'%s %s "%s" %s' % (self.treatment, self.first_name,
                                       self.nickname, self.surname)
        return u'%s %s %s' % (self.treatment, self.first_name, self.surname)

    def site_link(self):
        if self.site_url:
            return u'<a href="%s">%s</a>' % (self.site_url,
                                             self.site_title or 'Site')
    site_link.short_description = 'Site'
    site_link.allow_tags = True


class Telephone(models.Model):
    KIND_CHOICES = (
        ('wk', 'work'),
        ('rd', 'residencial'),
        ('cp', 'cellphone'),
        ('ot', 'other'),
    )
    kind = models.CharField(max_length=2, choices=KIND_CHOICES)
    number = models.CharField(max_length=20)

    call_from = models.TimeField(default=MIN_TIME)
    call_until = models.TimeField(default=MAX_TIME)

    contact = models.ForeignKey(Contact, related_name='secondary_telephones')

    def __unicode__(self):
        kinds = dict(self.KIND_CHOICES)
        return u'%s/%s: %s' % (self.contact, kinds[self.kind], self.number)

    def ok_to_call_now(self):
        now = datetime.datetime.now().time()
        start, end = self.call_from, self.call_until
        if start < now < end:
            return True
        return False
    ok_to_call_now.boolean = True
    ok_to_call_now.short_description = 'Ok to call now?'


class Email(models.Model):
    KIND_CHOICES = (
        ('wk', 'work'),
        ('pr', 'personal'),
        ('ot', 'other'),
    )
    kind = models.CharField(max_length=2, choices=KIND_CHOICES)
    address = models.EmailField('Email Address', max_length=50)
    contact = models.ForeignKey(Contact, related_name='secondary_emails')

    def __unicode__(self):
        kinds = dict(self.KIND_CHOICES)
        return u'%s/%s: %s' % (self.contact, kinds[self.kind],
                               self.address)


class City(models.Model):
    name = models.CharField(max_length=50)
    state_abbrev = models.CharField('State (abbreviated)', max_length=5,
                                    blank=True)
    country = models.CharField(max_length=50)

    def __unicode__(self):
        return u'%s (%s/%s)' % (self.name, self.state_abbrev, self.country)

    class Meta:
        verbose_name_plural = 'Cities'


class Address(models.Model):
    number = models.IntegerField(null=True, blank=True)
    complement = models.CharField(max_length=10, blank=True)
    street = models.CharField(max_length=50)
    neighbourhood = models.CharField(max_length=50)
    city = models.ForeignKey(City)

    class Meta:
        verbose_name_plural = 'Addresses'

    def __unicode__(self):
        number = str(self.number)
        if self.complement:
            number += ' ' + self.complement
        return u', '.join([number, self.street, unicode(self.city)])


class ContactAddress(models.Model):
    KIND_CHOICES = (
        ('wk', 'work'),
        ('dc', 'domicile'),
        ('vc', 'vacation'),
        ('ot', 'other'),
    )
    kind = models.CharField(max_length=2, choices=KIND_CHOICES)
    contact = models.ForeignKey(Contact)
    address = models.ForeignKey(Address)

    class Meta:
        verbose_name = "Contact's Address"
        verbose_name_plural = "Contact's Addresses"

    def __unicode__(self):
        kinds = dict(self.KIND_CHOICES)
        return '%s/%s: %s' % (self.contact, kinds[self.kind], self.address)
